"""migrate datastore data

修订 ID: ade8cdca5470
父修订: 557fef3a156f
创建时间: 2023-10-11 19:57:51.023847

"""
from __future__ import annotations

from collections.abc import Sequence

import sqlalchemy as sa
from alembic.op import run_async
from nonebot import logger, require
from sqlalchemy import Connection, inspect
from sqlalchemy.ext.asyncio import AsyncConnection, AsyncSession

revision: str = "ade8cdca5470"
down_revision: str | Sequence[str] | None = "557fef3a156f"
branch_labels: str | Sequence[str] | None = None
depends_on: str | Sequence[str] | None = None


def _has_table(conn: Connection, table_name: str) -> bool:
    insp = inspect(conn)
    return table_name in insp.get_table_names()


async def data_migrate(conn: AsyncConnection):
    from nonebot_plugin_datastore.db import get_engine

    async with get_engine().connect() as ds_conn:
        has_table = await ds_conn.run_sync(
            _has_table, "nonebot_plugin_wordcloud_schedule"
        )
        if not has_table:
            logger.info("wordcloud: 未发现来自 datastore 的数据")
            return

    async with AsyncSession(get_engine()) as ds_sess:
        # 读取数据
        result = await ds_sess.execute(
            sa.text("SELECT * FROM nonebot_plugin_wordcloud_schedule")
        )
        data = result.all()

    if not data:
        return

    # 写入数据
    logger.info("wordcloud: 发现来自 datastore 的数据，正在迁移...")
    await conn.execute(
        sa.text(
            "INSERT INTO nonebot_plugin_wordcloud_schedule (id, target, time) VALUES (:id, :target, :time)"
        ),
        [{"id": i, "target": t, "time": ti} for i, t, ti in data],
    )
    logger.info("wordcloud: 迁移完成")


def upgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    try:
        require("nonebot_plugin_datastore")
    except RuntimeError:
        return

    run_async(data_migrate)
    # ### end Alembic commands ###


def downgrade(name: str = "") -> None:
    if name:
        return
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
